;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================

ORG 4FD2			;20434

@DFile=400C			;Address of display file
@LastK=4025			;Keyboard debounce value
@Frames=4034			;16436, Frame counter
@Coords=4036			;16438, PLOT coords
@Cell=403C			;16444, Cell character(s)
@BKG=403D			;16445, Background character
@Density=403E			;16446, Density value
@LoopFlag=4083			;16515, Player On/OFF flag
@LoadScr=4CC5			;19653, Load main form
@Scratch=BCE0			;Scratch pad area

;================================
; Jump Table
;================================
	JP RandLoop		;20434 Random Screen
	JP WallPaper		;20437 Wallpaper
	JP Kaleid01		;20440 Kaleidoscope Style 1
	JP Kaleid02		;20443 Kaleidoscope Style 2
	JP Kaleid03		;20446 Pixel Kaleidoscope
	JP PxlMandala		;20449 Pixel Mandala
	JP ZXMandala		;20452 ZX81 Mandala
	JP Maze			;20455 Endless maze
;================================
; Random screen loop
;================================
@RandLoop
	CALL SaveDF		;Save main form
	CALL Rnd8Bit		;Get 8bit random number, 0 TO 255
	LD (Rand01+1),A		;Set density value
	AND 01
	JR Z +2			;0, move on
	LD A,80			;1, "g "
	LD (Rand03+1),A		;Set BKG, " " or "g "
	CALL PikChr		;Get chr to print
	LD B,A			;B=Chr to print
	LD A,(Rand03+1)		;A=0 or 128
	ADD A,B			;A=Chr to print
	LD (Rand02+1),A		;Set chr to print
@Outer
	LD HL,Scratch		;Work area
	LD BC,0318		;Byte count
@Inner
	PUSH HL			;PUSH DFile address
	CALL Rnd8Bit		;Get 8bit random number, 0 TO 255
	LD L,A			;L=range 0 to 255
@Rand01
	LD A,00			;Density value
	CP L			;L>Density?
@Rand02
	LD A,00			;Chr value
	JR C +2			;Print chr if CY
@Rand03
	LD A,00			;Print bkg if not CY
	POP HL			;POP DFile address
	BIT 6,(HL)		;N/L?
	JR NZ +1		;Yes, skip print
	LD (HL),A		;Print chr or bkg
	INC HL			;Next DFile address
	DEC BC 			;Decrease counter
	LD A,B 
	OR C 
	JR NZ Inner		;Loop until BC=0
	CALL LoadDF
	CALL Stopped
	JR Outer
;================================
@SaveDF
	LD HL,(DFile)
	INC HL
	LD DE,Scratch
	JR MoveDF
@LoadDF
	LD HL,Scratch
	LD DE,(DFile)
	INC DE
@MoveDF
	LD BC,0318
	LDIR
	RET
;================================
; Wallpaper Loop
;================================
@WallPaper
	LD A,18			;Delay value for this routine
	LD (Keytest+2),A	;Set delay value
	CALL SaveDF
@WallLoop
	CALL Rnd8Bit		;Get 8bit random number, 0 TO 255
	AND 01
	LD C,A
	JR Z +2			;0, move on
	LD C,80			;1, C=128 for graphic chrs
	LD HL,Cell		;Pattern data address
	LD B,07			;Byte count
	PUSH HL			;PUSH Pattern address
	LD E,0A			;Set upper limit
	CALL Modulus		;A=0 to 9
	POP HL			;POP Pattern address
	ADD A,C			;Add 0 or 128 to A
	LD (HL),A		;Store chr code
	INC HL			;Increase pattern addr
	DJNZ -12		;Loop until B=0
	XOR A			;Initialize counter
	LD DE,Scratch		;DE=work area
	LD HL,Cell		;Source
	LD C,07			;Byte count
	LDIR			;Copy pattern to scratch pad
	INC A			;Increase counter
	CP 71			;Upper limit
	JR NZ -12		;Repeat 113 times
	LD B,18			;Restore N/L chrs
	LD HL,Scratch
	DEC HL
	LD DE,0021		;Row count
	ADD HL,DE		;Next row
	LD (HL),76		;Set N/L
	DJNZ -5			;Loop until B=0
	CALL LoadDF
	CALL Stopped		;Loop until keypressed
	JR WallLoop
;================================
; Kaleidoscope 01
;================================
@Kaleid01
	CALL SetDensity		;Set density range to 87 to 150
	CALL CLSBW		;CLS with " " or "g "
	CALL PikChr		;Pick a chr to print
@KLoop01
	LD E,0C			;Set upper limit, 12
	CALL Modulus
	LD B,A          	;B=Ycoord
	LD E,10			;Set upper limit, 16
	CALL Modulus
        LD C,A          	;C=Xcoord
	CALL PrintCell		;Get chr to print, (BKG) or (Cell)
	CALL PrintQuads		;Print chr to the four quadrants
	CALL Stopped		;Get key press
	JR KLoop01		;No, loop 
;================================
; Kaleidoscope 02
;================================
@Kaleid02
	CALL SetDensity		;Set density range to 87 to 150
	CALL CLSBW		;CLS with " " or "g "
	CALL RND8Bit
	AND 01			;Odd or Even?
	LD A,08
	JR Z +5
	LD A,(BKG)
	ADD A,80
	LD (BKG),A		;Set clear chr 00,08 or 80
	CALL PikChr		;Pick a chr to print
@KLoop02
	CALL PrintCell		;Get chr to print, (BKG) or (Cell)
	PUSH AF			;PUSH chr to print
	LD E,10			;Set upper limit, 16
	CALL Modulus
	LD B,A			;Set Ycoord
	CALL Modulus
	LD C,A			;Set Xcoord
	POP AF			;POP chr to print
	PUSH BC
	CALL PrintQuads		;Print to the 4 quadrants
	POP BC			;POP Y/Xcoords
	LD A,B			;Swap Y/Xcoords
	LD B,C
	LD C,A
	CALL PrintQuads		;Print to the 4 quadrants
	CALL Stopped		;Test keys
        JR KLoop02
;================================
; Kaleidoscope By S.H. McGurrin
; Syntax v5,n01 p.19
;================================
@Kaleid03
	CALL Release		;Wait for no key press
	CALL CLSBW		;CLS with " " or "g "
@KLoop03
	LD E,3F			;Set upper limit
	CALL Modulus		;Get Xcoord, range=0 to 63
	LD C,A
	LD E,2F			;Set upper limit
	CALL Modulus		;Get Ycoord, range=0 to 47
	LD B,A			;BC=Y/Xcoords
	CALL Rnd8Bit		;Get 8bit random number, 0 TO 255
	AND 01
        JR Z +2			;Jump if even
        LD A,40            	;40 FOR Plot, 0 FOR UNPlot
	PUSH AF		   	;PUSH Plot/UnPlot Flag
	CALL Plot		;PLOT/UNPLOT X,Y
        LD A,2F			;47
        SUB B
        LD B,A
	POP AF 		   	;POP Plot/UnPlot Flag
	PUSH AF		   	;PUSH Plot/UnPlot Flag
	CALL Plot		;PLOT/UNPLOT X,47-Y
        LD A,3F			;63
        SUB C
        LD C,A
	POP AF		   	;POP Plot/UnPlot Flag
	PUSH AF		   	;PUSH Plot/UnPlot Flag
	CALL Plot		;PLOT/UNPLOT 63-X,47-Y
        LD A,2F			;47
        SUB B
        LD B,A
	POP AF		 	;POP Plot/UnPlot Flag
	CALL Plot		;PLOT/UNPLOT 63-X,Y
	CALL Stopped
        JR KLoop03
;================================
; ZX81 Mandala
;================================
@ZXMandala
	CALL CLSBW		;CLS with " " or "g "
	LD A,(BKG)		;Get BKG value
	LD (MandSet+1),A	;Set BKG value
	CALL PikChr		;Get cell chr
	LD (MandSet+5),A	;Set cell chr
	CALL Rnd8Bit		;Get random number
	AND 01
	JR NZ +4		;1, print cell chr
@MandSet
	LD A,00			;0, print BKG chr
	JR +2
	LD A,00			;Print cell chr
	LD (Setchr+1),A		;Set chr to print
	LD E,15			;Set upper limit
	CALL Modulus
	LD B,A			;B=0 to 21
	LD (Coords+1),A		;Save coord
	CALL Modulus
	LD C,A			;C=0 to 21
	LD (Coords),A		;Save coord
	LD A,08
	ADD A,C
	LD C,A
	CALL Print		;B,C+8
	LD E,C
	LD A,08
	ADD A,B
	LD C,A
	LD B,E
	CALL Print		;C,B+8
	LD E,C
	LD A,18
	SUB B
	LD C,A
	LD B,E
	CALL Print		;C,24-B
	LD A,18
	SUB C
	LD C,A
	CALL Print		;B,24-C
	LD A,17
	SUB B
	LD B,A
	LD A,08
	ADD A,C
	LD C,A
	CALL Print		;23-B,C+8
	LD E,B
	LD A,17
	SUB C
	LD B,A
	LD A,08
	ADD A,E
	LD C,A
	CALL Print		;23-C,B+8
	LD E,B
	LD A,17
	SUB C
	LD B,A
	LD A,18
	SUB E
	LD C,A
	CALL Print		;23-C,24-B
	LD A,17
	SUB B
	LD B,A
	LD A,18
	SUB C
	LD C,A
	CALL Print		;23-B,24-C
	CALL Stopped		;Check keys
	JR MandSet-7		;Loop until "Q" pressed
;================================
; Calculate print position then print chr
;  B=Ycoord
;  C=Xcoord
;================================
@Print
	LD HL,0000		;Set accumulator to 0
	LD DE,0021		;Bytes/row
	LD A,B
	AND A			;Row=0?
	JR Z +3			;Yes, move on
	ADD HL,DE		;HL=33*row value
	DJNZ -3
	ADD HL,BC		;HL=33*row value+column offset
	LD BC,(DFile)		;Get DFile
	INC BC			;Skip 1st N/L chr
	ADD HL,BC		;HL=print position
@Setchr
	LD (HL),00		;PRINT AT B,C;chr
	LD BC,(Coords)		;Restore coords
	RET
;================================
; Endless Maze
;================================
@Maze
	LD A,0B			;Delay value for this routine
	LD (KeyTest+2),A	;Set delay value
@MazeLoop
	CALL ScrollUp
	LD C,20
	SBC HL,BC		;Start of line 24
	PUSH HL			;PUSH print location
	CALL RND8Bit		;Get random number
	AND 0F			;Range 0 to 15
	LD HL,MazeChr		;Chr list
	LD C,A			;BC=Location of chr in list
	ADD HL,BC		;Point to chr addr
	LD A,(HL)		;Get chr
	POP HL			;POP print location
	LD (HL),A		;Print chr
	INC HL			;Next print location
	BIT 6,(HL)		;Test for N/L chr
	JR Z -19		;Print next chr
	CALL Stopped		;Loop until "Q"uit pressed
	JR MazeLoop		;No, loop until keypressed
;================================
@MazeChr			;Maze chrs
	80 01 02 87 04 05 83 03 85 81 82 07 84 06 86 00
;================================
; 64x48 Plot Routine
;
;  B=YCoord, 0 to 43
;  C=XCoord, 0 to 63
;  A=40H/64=PLOT
;  A=00=UNPLOT
;===============================
@Plot
	PUSH BC			;PUSH X,Y coords
	PUSH AF			;PUSH Plot/UnPlot flag
	LD A,87			;A=87=NW Pixel
	BIT 0,B			;Y coord
	JR NZ +2		;Odd line number?
	ADD A,10		;A=97=SW Pixel 
	BIT 0,C			;X coord
	JR Z +2			;Even col number?
	ADD A,08		;A=8F=NE Pixel or A=9F=SE Pixel
	LD D,A			;D=Pixel quadrant value
	POP AF			;POP Plot/UnPlot flag
	ADD A,D			;A=D IF UNPlot or D+64 IF Plot
	LD (PlotPX+1),A		;Self modifying code
	SRL B			;Convert Y-coord to Print coord, Y/2
	SRL C			;Convert X-coord to Print coord, X/2
	LD HL,(DFile)		;Get DFile
	INC HL			;Start of DFile
	LD D,00
	LD E,C			;DE=Column number
	ADD HL,DE		;HL=Column position in DFile
	LD A,17			;23-Y/2
	SUB B
	LD B,A			;B=Row number
	JR Z +6
	LD DE,0021		;Bytes/row
	ADD HL,DE
	DJNZ -3
	LD A,(HL)		;Get chr at DFile addr
	CP 08			;LO Pixel Chr
	JR C PlotPX
	CPL			;HI Pixel Chr
@PlotPX				
	SET 00,A		;00=SR SET/RES X,A
	BIT 3,A
	JR Z +5
	CPL
	AND 8F
	JR +2
	AND 0F			;Low value PIXEL, codes 1 to 7
	LD (HL),A		;Hi value PIXEL, codes 128 to 135
	POP BC			;POP X,Y coords
	RET
;================================
; Pixel Mandala
;================================
@PxlMandala
	CALL CLSBW		;CLS with "g " or " "
	BIT 7,D			;Test CLS chr
	JR Z Mandala		;Skip next section if D was 0
	LD BC,0020		;Print last line
	SBC HL,BC
	DEC C
	LD (HL),03		;Pixel line chr
	LD D,H
	LD E,L
	INC DE
	LDIR
@Mandala
	LD A,40			;Plot flag
	LD (MandPlot+1),A	;Set for plot
	LD B,02 
@MandLoop
	PUSH BC 		;PUSH counter
	LD E,2B			;Set upper limit to 45
	CALL Modulus		;Get Xcoord, range 0 to 45
	LD D,A			;Xcoord
	PUSH DE 		;PUSH Xcoord
	CALL Modulus		;Get Ycoord, range 0 to 45
	POP DE 			;POP Xcoord
	LD E,A 			;DE=XYcoords
	LD (Coords),DE		;Store coords
	CALL QuadPlot		;Plot quads
	XOR A			;Unplot flag
	LD (MandPlot+1),A	;Set for unplot
	POP BC 			;POP counter
	DJNZ MandLoop		;Loop back
	CALL Stopped
	JR Mandala
;================================
; Mandala Plot Quads
;  DE=Initial coords
;================================
@QuadPlot
	LD A,0C
	ADD A,D
	LD C,A
	LD B,E
	CALL MandPlot		;D+12,E
	LD A,0C
	ADD A,E
	LD C,A
	LD B,D
	CALL MandPlot		;E+12,D
	LD A,0C
	ADD A,E 
	LD C,A
	LD A,28 
	SUB D 
	LD B,A
	CALL MandPlot		;E+12,40-D
	LD A,0C 
	ADD A,D
	LD C,A
	LD A,28 
	SUB E 
	LD B,A
	CALL MandPlot		;D+12,40-E
	LD A,34 
	SUB D 
	LD C,A
	LD B,E
	CALL MandPlot		;52-D,E
	LD A,34 
	SUB E 
	LD C,A
	LD B,D
	CALL MandPlot		;52-E,D
	LD A,34 
	SUB E 
	LD C,A
	LD A,28 
	SUB D 			
	LD B,A
	CALL MandPlot		;52-E,40-D
	LD A,34 
	SUB D 
	LD C,A
	LD A,28 
	SUB E 
	LD B,A
	CALL MandPlot		;52-D,40-E
	RET
;================================
; PLOT Mandala Quads
;================================
@MandPlot
	LD A,00			;PLOT flag
	INC B			;Increase Ycoord before plotting
	INC B
	INC B
	INC B
	CALL Plot
	LD DE,(Coords)	 	;Restore coords
	RET
;================================
; Set density value range 87 to 150
;================================
@SetDensity
	CALL Release		;Wait for key release
	CALL Rnd8Bit		;Get 8bit random number, 0 TO 255
	AND 3F			;Range 0 to 63
	LD B,A			;B=0 to 63
	LD A,96			;A=150
	SUB B			;A=150-B, range 87 to 150
	LD (Density),A		;Set density value
	RET
;================================
; Scroll Up
;================================
@ScrollUp
	LD HL,(DFile)
	PUSH HL 
	LD DE,0021 
	ADD HL,DE 
	POP DE 
	LD BC,02F7 
	LDIR
	RET
;================================
; Get random cell chr
;================================
@PikChr
	CALL Rnd8Bit		;Get 8bit random number, 0 TO 255
	AND 0F			;Range 0 to 15
	LD B,00
	LD C,A			;BC=Chr number
	LD HL,ChrsList		;Cell chrs list
	ADD HL,BC		;HL=chr address
	LD A,(HL)		;Get chr
	BIT 3,(IY+61)		;Grey BKG?
	JR NZ +6		;Jump if grey BKG
	BIT 7,(IY+61)		;"g " BKG?
	JR Z +2			;Jump if not "g " BKG
	ADD A,80		;Add 128
	CP (IY+60)		;Loop back if same as last cell
	JR Z PikChr
	CP (IY+61)		;Loop back if same as BKG
	JR Z PikChr
	LD (Cell),A		;Set chr
	RET
;================================
@ChrsList			;Cell chr list
	01 03 05 06 07 08 09 15 16 17 1B 1C 3B 3D 80
;================================
; Set print chr to (BKG) or (Cell)
; based on random number generated.
;================================
@PrintCell
	CALL Rnd8Bit		;Get 8bit random number, 0 TO 255
	LD L,A			;L=8-Bit random number, range 0 to 255
	LD A,(Density)		;Get density value
	CP L			;L>Density?
	LD A,(Cell)		;Get chr to print
	JR C +3			;Yes, print cell
	LD A,(BKG)		;No, print bkg
	LD (PokeA+1),A		;POKE chr to print
	RET
;================================
; Print A to the four quadrants
; of the screen.
;================================
@PrintQuads
	CALL PrintX		;Print to NW quadrant B,C
	LD A,1F
        SUB C
        LD C,A
	CALL PrintX		;Print to NE quadrant B,31-C
	LD A,17
        SUB B
        LD B,A
	CALL PrintX		;Print to SE quadrant 25-B,31-C
	LD A,1F			
        SUB C
	LD C,A
;================================
; Locate print position in DFile
; then print POKEd chr.
;================================
@PrintX
	PUSH DE
	PUSH BC
	INC B			;Ycoord must be > 0
	LD HL,(DFile)		;Get start of DFile
	INC HL
	LD DE,0000		;Initialize DE
	ADD HL,DE		
	LD E,21			;DE=33 bytes/row
	DJNZ -5			;Add Bx33 to start of DFile
	LD E,C			;DE=Xcoord offset
	ADD HL,DE		;Add X offset to DE
@PokeA
	LD (HL),00		;POKEd chr to print
	POP BC			;POP Y/Xcoords
	POP DE
	RET
;================================
; Wait for key release
;================================
@Release
	LD A,(LastK)		;Check for keypress
	INC A
	JR NZ Release		;Loop until key released
	RET
;================================
; If tune ended or keypressed then
; return to BASIC
;================================
@Stopped			;First test stopped bit
	LD HL,LoopFlag		;Loopflag addr
	BIT 7,(HL)		;Test stopped bit.
	JR Z KeyTest		;If 0 move on, still playing
	RES 7,(HL)		;Tune ended, reset stopped flag
	BIT 0,(HL)		;Test loop flag
	JR NZ TuneDone		;If NZ end video
@KeyTest			;Test for keypress
	LD BC,0001 		;Delay Loop
	DEC BC 			;Delay
	LD A,B 
	OR C 
	JR NZ -5
	LD A,(LastK)		;Check for keypress
	INC A
	RET Z			;Return if no keypress
	XOR A
	LD (KeyTest+2),A	;Reset delay value
	POP BC			;Clear stack
	JR Release		;Return via Release
@TuneDone
	CALL LoadScr		;Restore screen
	POP BC			;Clear the stack
	CALL Release		;Wait for key release
	LD BC,+90		;
	RET			;GOTO 90
;================================
; Clear screen with random " " or "g "
;================================
@CLSBW
	CALL Rnd8Bit		;Get 8bit random number, 0 TO 255
	AND 01
	JR Z +2			;0, move on
	LD A,80			;1, "g "
;================================
; Clear screen with chr A
;================================
@CLS
	LD (BKG),A		;Set BKG chr
	LD D,A			;CLS with BKG chr
	LD HL,(DFile)
	LD BC,0318		;Byte count
	INC HL
	BIT 6,(HL)		;N/L?
	JR NZ +1		;Yes, skip it
	LD (HL),D		;Print BKG chr.
	DEC BC 			;Decrease counter
	LD A,B 
	OR C 
	JR NZ -11		;Loop until BC=0
	RET
;================================
; 8-Bit Random Number Generator
; Reg BC not used
; On exit...
;  Reg A=random number in the
;        range 0 to 255
;================================
@Rnd8Bit
	LD DE,0000		;Seed
	LD HL,(Frames)     	;Create pointer to ROM address
	DEC DE	
	ADD HL,DE
	LD A,H
	AND 1F
	LD H,A			;Limit address to <=8192
	LD (Rnd8Bit+1),HL	;New seed
	LD A,(HL)		;Get value at ROM address
	RET
;================================
; Modulus code to ensure 
; number in desired range.
; On entry...
;  E=Upper Limit
; Reg BC not used
; On exit...
;  A=random number between 0 and
;    upper limit
;================================
@Modulus
	LD D,00
	INC E			;DE=dividend
	PUSH DE
	CALL Rnd8Bit		;Get 8-Bit random number
	POP DE
	LD H,D			
	LD L,A			;HL=8-Bit random number
@ModLoop
	OR A			;Clear CY
	SBC HL,DE		;Subtract dividend from HL
	JP P ModLoop		;Keep doing it till negative
	ADD HL,DE		;Add the divided back again
	LD A,L			;Return results in A
	DEC E
	RET